<html>
    <head>
        <meta name="viewport" content="width=device-width"></meta>
        <title>{{ ref.key }}</title>
    </head>
    <body>
        <pre>{{ ref.content }}</pre>
    </body>
</html>
