{% extends "!theme/page.html" %}

{% from "bibentry.html" import bibentry, bibcss %}

{#
TODO
    - fix issue with extra period in title if title ends in punctuation
#}

{% block after_content %}
<h2>Publications</h2>
{% for ref in bibliography %}
{{ bibentry(ref) }}
{% endfor %}
{% endblock %}

{% block meta %}
{{ super() }}
<style>
{{ bibcss }}
</style>
{% endblock %}
